#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: config.py
# Author: Haoyu Lu
# Mail: lhy1998@ruc.edu.cn
# Created Time:  2022-3-18 19:17:34
#############################################

from pathlib import Path

import yaml
from easydict import EasyDict


def log_config_to_file(cfg, pre='cfg', logger=None):
    for key, val in cfg.items():
        if isinstance(cfg[key], EasyDict):
            logger.info('\n%s.%s = edict()' % (pre, key))
            log_config_to_file(cfg[key], pre=pre + '.' + key, logger=logger)
            continue
        logger.info('%s.%s: %s' % (pre, key, val))


def cfg_from_list(cfg_list, config):
    """Set config keys via list (e.g., from command line)."""
    from ast import literal_eval
    assert len(cfg_list) % 2 == 0
    for k, v in zip(cfg_list[0::2], cfg_list[1::2]):
        key_list = k.split('.')
        d = config
        for subkey in key_list[:-1]:
            assert subkey in d, 'NotFoundKey: %s' % subkey
            d = d[subkey]
        subkey = key_list[-1]
        assert subkey in d, 'NotFoundKey: %s' % subkey
        try:
            value = literal_eval(v)
        except:
            value = v

        if type(value) != type(d[subkey]) and isinstance(d[subkey], EasyDict):
            key_val_list = value.split(',')
            for src in key_val_list:
                cur_key, cur_val = src.split(':')
                val_type = type(d[subkey][cur_key])
                cur_val = val_type(cur_val)
                d[subkey][cur_key] = cur_val
        elif type(value) != type(d[subkey]) and isinstance(d[subkey], list):
            val_list = value.split(',')
            for k, x in enumerate(val_list):
                val_list[k] = type(d[subkey][0])(x)
            d[subkey] = val_list
        else:
            assert type(value) == type(d[subkey]), \
                'type {} does not match original type {}'.format(type(value), type(d[subkey]))
            d[subkey] = value


def merge_new_config(config, new_config):
    if '_BASE_CONFIG_' in new_config:
        with open(new_config['_BASE_CONFIG_'], 'r') as f:
            try:
                yaml_config = yaml.load(f, Loader=yaml.FullLoader)
            except:
                yaml_config = yaml.load(f)
        config.update(EasyDict(yaml_config))

    for key, val in new_config.items():
        if not isinstance(val, dict):
            config[key] = val
            continue
        if key not in config:
            config[key] = EasyDict()
        merge_new_config(config[key], val)

    return config


def cfg_from_yaml_file(cfg_file, config):
    with open(cfg_file, 'r') as f:
        try:
            new_config = yaml.load(f, Loader=yaml.FullLoader)
        except:
            new_config = yaml.load(f)

        merge_new_config(config=config, new_config=new_config)

    return config


cfg = EasyDict()
cfg.ROOT_DIR = (Path(__file__).resolve().parent / '../').resolve()
# cfg.LOCAL_RANK = 0



import torch
import torch.nn as nn


class FakeTransformer(nn.Module):

    def __init__(self, input_dim, hidden_dim, output_dim):
        super(FakeTransformer, self).__init__()

        self.fc1 = nn.Linear(input_dim, hidden_dim)
        self.dropout1 = nn.Dropout(0.5)
        self.relu = nn.ReLU()
        self.fc2 = nn.Linear(hidden_dim, output_dim)

    def forward(self, x):
        out = self.fc1(x)
        
        out = self.relu(out)
        out = self.fc2(out)
        return out



from .vl_model import VL_model

import torch

__all__ = {
    'VL': VL_model
}


def build_network(model_cfg=None, args=None):
    model = __all__[model_cfg.NAME](
        model_cfg=model_cfg,
        args=args
    )
    return model



import torch
import torch.nn as nn
from transformers import AutoModel


class Bert(nn.Module):

    def __init__(self, cfg):
        super(Bert, self).__init__()
        self.cfg = cfg
        self.bert =  AutoModel.from_pretrained(cfg.ENCODER) 

    def forward(self, x): 
        y = self.bert(x, return_dict=True).last_hidden_state
        return y



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: Haoyu Lu
# Mail: lhy1998@ruc.edu.cn
# Created Time:  2022-3-18 19:17:34
#############################################

import os
 
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as readme:
    README = readme.read()
 
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
 
# setuptools.setup(
#     name="example-pkg-YOUR-USERNAME-HERE", # 库名，需要在pypi中唯一
#     version="0.0.1",                       # 版本号
#     author="Example Author",               # 作者
#     author_email="author@example.com",     # 作者邮箱（方便使用者发现问题后联系我们）
#     description="A small example package", # 简介
#     long_description=long_description,              # 详细描述（一般会写在README.md中）
#     long_description_content_type="text/markdown",  # README.md中描述的语法（一般为markdown）
#     url="https://github.com/pypa/sampleproject",   # 库/项目主页，一般我们把项目托管在GitHub，放该项目的GitHub地址即可
#     packages=setuptools.find_packages(),    #默认值即可，这个是方便以后我们给库拓展新功能的
#     classifiers=[                    # 指定该库依赖的Python版本、license、操作系统之类的
#         "Programming Language :: Python :: 3",
#         "License :: OSI Approved :: MIT License",
#         "Operating System :: OS Independent",
#     ],
#     install_requires=[               # 该库需要的依赖库
#         '你的库依赖的第三方库（也可以指定版本）',
#         # exapmle
#         'pyautogui',
#         'Django >= 1.11, != 1.11.1, <= 2',
#     ],
#     python_requires='>=3.6',
# )

from setuptools import setup, find_packages            #这个包没有的可以pip一下

setup(
    name = "wenlan_video_public",      #这里是pip项目发布的名称
    version = "1.0.0",  #版本号，数值大的会优先被pip
    keywords = ("pip", "wenlan"),
    description = "Wenlan - Large scale cross-modal pre-training model",
    long_description = "A large-scale video-language pre-training model.",
    license = "MIT Licence",

    url = "https://github.com/rucmlcv/Wenlan-Video-Public",     #项目相关文件地址，一般是github
    author = "Haoyu Lu",
    author_email = "lhy1998@ruc.edu.cn",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = ["numpy"],        #这个项目需要的第三方库

    classifiers=[                   
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: vl_model.py
# Author: Haoyu Lu
# Mail: lhy1998@ruc.edu.cn
# Created Time:  2022-3-18 19:17:34
#############################################

import torch
import torch.nn as nn
import torchvision
from transformers import AutoTokenizer
from .fakeTransformer import FakeTransformer
from .bert import Bert


import torch.nn.functional as F
import timm

import numpy as np
import math




class ImgLearnableEncoder(nn.Module):
    def __init__(self, model_cfg, args):
        super(ImgLearnableEncoder, self).__init__()

        
        self.backbone = timm.create_model(model_cfg.CNN, pretrained=False)

        self.model_cfg = model_cfg
        self.learnable = nn.ModuleDict()

        img_encoder_layer = nn.TransformerEncoderLayer(d_model=self.model_cfg.IMG_FEATURE_DIM, nhead=self.model_cfg.IMG_TRANSFORMER_HEAD)
        self.learnable['imgAtt'] = nn.TransformerEncoder(img_encoder_layer, num_layers=self.model_cfg.IMG_TRANSFORMER_LAYER)
        self.learnable['imgFC'] = FakeTransformer(model_cfg.IMG_FEATURE_DIM, model_cfg.HIDDEN_DIM_1, model_cfg.HIDDEN_DIM_2)
        self.learnable['max_pool'] = nn.Sequential(nn.AvgPool2d(model_cfg.ROI_GRID_SIZE, stride=1)) 

        # self.learnable['final_mlp'] = FakeTransformer(model_cfg.HIDDEN_DIM_2*2, model_cfg.HIDDEN_DIM_2, model_cfg.HIDDEN_DIM_2)


    def roi_grid_pool(self, spatial_features_2d, rois):
        """
        Args:
            rois: (B, num_rois, 4)
            spatial_features_2d: (B, C, H, W)
        Returns:
            pooled_features : (B, num_rois, C)
        """
        batch_size = spatial_features_2d.size(0)
        rois = rois.detach()
        height, width = spatial_features_2d.size(2), spatial_features_2d.size(3)
        down_sample_ratio = self.model_cfg.IMG_SIZE / height
        pooled_features_list = []
        torch.backends.cudnn.enabled = False
        for b_id in range(batch_size):
            # Map global boxes coordinates to feature map coordinates
            x1 = rois[b_id, :, 0] / down_sample_ratio
            y1 = rois[b_id, :, 1] / down_sample_ratio
            x2 = rois[b_id, :, 2] / down_sample_ratio
            y2 = rois[b_id, :, 3] / down_sample_ratio
            angle = torch.zeros((1), device=spatial_features_2d.device)
            cosa = torch.cos(angle)
            sina = torch.sin(angle)

            theta = torch.stack((
                (x2 - x1) / (width - 1) * cosa, (x2 - x1) / (width - 1) * (-sina), (x1 + x2 - width + 1) / (width - 1),
                (y2 - y1) / (height - 1) * sina, (y2 - y1) / (height - 1) * cosa, (y1 + y2 - height + 1) / (height - 1)
            ), dim=1).view(-1, 2, 3).float()

            grid_size = self.model_cfg.ROI_GRID_SIZE
            grid = nn.functional.affine_grid(
                theta,
                torch.Size((rois.size(1), spatial_features_2d.size(1), grid_size, grid_size))
            )

            pooled_features = nn.functional.grid_sample(
                spatial_features_2d[b_id].unsqueeze(0).expand(rois.size(1), spatial_features_2d.size(1), height, width),
                grid
            )
            pooled_features = self.learnable['max_pool'](pooled_features)
            pooled_features_list.append(pooled_features.squeeze())

        torch.backends.cudnn.enabled = True
        pooled_features = torch.stack(pooled_features_list, dim=0)

        return pooled_features

    def forward(self, imgFea, image_boxs):
        imgFea = self.backbone.forward_features(imgFea)
        imgFea = self.roi_grid_pool(imgFea, image_boxs)
        imgFea = F.normalize(imgFea, p=2, dim=-1)

        imgFea = self.learnable['imgAtt'](imgFea.transpose(0, 1)).transpose(0,1)    # TODO
        imgFea = imgFea.mean(1)
        imgFea = self.learnable['imgFC'](imgFea)


        return imgFea


class TextLearnableEncoder(nn.Module):
    def __init__(self, model_cfg):
        super(TextLearnableEncoder, self).__init__()

        self.backbone = Bert(model_cfg)
        self.model_cfg = model_cfg      # TODO: model_cfg

        self.learnable = nn.ModuleDict()
        
        text_encoder_layer = nn.TransformerEncoderLayer(d_model=model_cfg.TEXT_FEATURE_DIM, nhead=model_cfg.TEXT_TRANSFORMER_HEAD)
        self.learnable['textAtt'] = nn.TransformerEncoder(text_encoder_layer, num_layers=model_cfg.TEXT_TRANSFORMER_LAYER)

        self.learnable['textFC'] = FakeTransformer(model_cfg.TEXT_FEATURE_DIM, model_cfg.HIDDEN_DIM_1, model_cfg.HIDDEN_DIM_2)


    def forward(self, textFea, maskTexts):
        textFea = self.backbone(textFea)
        textFea = F.normalize(textFea, p=2, dim=-1)
        if self.model_cfg.TEXT_TRANSFORMER_LAYER != 0:
            textFea = self.learnable['textAtt'](textFea.transpose(0, 1), src_key_padding_mask=(maskTexts == 0)).transpose(0,1) 
        tmpMask = torch.where(maskTexts == 1, torch.tensor([1.0], device=maskTexts.device),
                              torch.tensor([0.0], device=maskTexts.device))
        textFea = (textFea * tmpMask.unsqueeze(-1)).sum(dim=1) / tmpMask.sum(dim=1).unsqueeze(-1)  # (bs, dim)
        textFea = self.learnable['textFC'](textFea)

        return textFea


class VL_model(nn.Module):

    def __init__(self, model_cfg, args):
        super(VL_model, self).__init__()

        self.model_cfg = model_cfg
        self.num_frame = model_cfg.num_frame

        self.learnable = nn.ModuleDict()
        self.learnable['imgencoder'] = ImgLearnableEncoder(model_cfg, args)
        self.learnable['textencoder'] = TextLearnableEncoder(model_cfg)
        
        video_layer = nn.TransformerEncoderLayer(d_model=model_cfg.HIDDEN_DIM_2, nhead=self.model_cfg.IMG_TRANSFORMER_HEAD)
        self.learnable['videoAtt'] = nn.TransformerEncoder(video_layer, num_layers=self.model_cfg.IMG_TRANSFORMER_LAYER)

        self.tokenizer = AutoTokenizer.from_pretrained('hfl/chinese-roberta-wwm-ext-large')

    def imgfea2videofea(self,t):
        t = t.reshape(-1, self.num_frame, t.shape[1])
        t = self.learnable['videoAtt'](t.transpose(0,1)).transpose(0,1)
        v = t.mean(dim = 1)
        return v       


    def forward(self, imgFea, image_boxs, texts, maskTexts):
        with torch.no_grad():
            imgFea = self.learnable['imgencoder'](imgFea, image_boxs) # <bsz, img_dim>
            imgFea = self.imgfea2videofea(imgFea)
            textFea = self.learnable['textencoder'](texts, maskTexts) # <bsz, img_dim>
            
            imgFea = F.normalize(imgFea, p=2, dim=-1)
            textFea = F.normalize(textFea, p=2, dim=-1)

            return [imgFea, textFea]


    def encode_video(self, imgFea, image_boxs):
        with torch.no_grad():
            imgFea = self.learnable['imgencoder'](imgFea, image_boxs) # <bsz, img_dim>
            imgFea = self.imgfea2videofea(imgFea)
            imgFea = F.normalize(imgFea, p=2, dim=-1)
            return imgFea

    def encode_text(self, texts, maskTexts):
        with torch.no_grad():
            textFea = self.learnable['textencoder'](texts, maskTexts) # <bsz, img_dim>
            textFea = F.normalize(textFea, p=2, dim=-1)
            return textFea



        




import os
import numpy as np
import torch
import torch.utils.data as data
import torchvision.transforms as transforms

import json
import random


import jsonlines
import argparse

from PIL import ImageFilter
from transformers import AutoTokenizer
from PIL import Image
from PIL import ImageFile


def visual_transforms_box(is_train=True, new_size=384):
    mean, std = [0.485, 0.456, 0.406], [0.229, 0.224, 0.225]
    normalize = transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225])

    return transforms.Compose([
        transforms.ToTensor(),
        transforms.Resize((new_size, new_size)),
        normalize])




def getLanMask(seq_lens, max_len):
    # seq_lens (bs)
    mask = torch.ones((seq_lens.size(0), max_len))  # (bs, max_len)
    idxs = torch.arange(max_len).unsqueeze(dim=0)  # (1, max_len)
    seq_lens = seq_lens.unsqueeze(-1)  # (bs, 1)
    mask = torch.where(idxs < seq_lens, mask, torch.Tensor([0.0]))
    return mask



class wenlan_transforms():
    def __init__(self):
        self.tokenizer = AutoTokenizer.from_pretrained('hfl/chinese-roberta-wwm-ext-large')
    
    def text_transform(self, text, max_length=50, device='cpu'):
        # text_transform = tokenizer #AutoTokenizer.from_pretrained('hfl/chinese-roberta-wwm-ext-large')
        text_info = self.tokenizer(text, padding='max_length', truncation=True,
                                        max_length=max_length, return_tensors='pt')
        text = text_info.input_ids.reshape(-1).to(device)
        text_len = torch.sum(text_info.attention_mask)
        textMask = getLanMask(text_len.unsqueeze(0), max_length).squeeze(0).to(device)

        return text, textMask
    
    def video_transform(self, video_path, device='cpu'):
        visual_transform = visual_transforms_box(False, 384)

        image_boxes = []
        images = []
        sorted_frames = os.listdir(video_path)
        for frame in sorted_frames:

            img_path = os.path.join(video_path, frame)
            image = Image.open(img_path).convert('RGB')
            image = visual_transform(image)
            images.append(image)
            img_box_s = []


            for grid_num in [1, 5]:
                for i in range(grid_num):
                    for j in range(grid_num):
                        img_box_s.append(torch.from_numpy(np.array(
                            [i * (384 / grid_num), j * (384 / grid_num), (i + 1) * (384 / grid_num),
                                (j + 1) * (384 / grid_num)])))
            image_boxes.append(torch.stack(img_box_s, 0))

        image_boxes = torch.stack(image_boxes, 0).to(device)
        images = torch.stack(images, 0).to(device)


        return images, image_boxes



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: load.py
# Author: Haoyu Lu
# Mail: lhy1998@ruc.edu.cn
# Created Time:  2022-3-18 19:17:34
#############################################


import argparse
import torch
import sys 
sys.path.append("..")

from models import build_network
from utils.config import cfg_from_yaml_file, cfg

def load_wenlan_model(load_checkpoint='../wenlan-video-model.pth', device='cpu'):
    parser = argparse.ArgumentParser()
    parser.add_argument('--cfg_file', type=str, default='cfg/moco_box.yml')

    args = parser.parse_args()
    cfg_from_yaml_file(args.cfg_file, cfg)

    cfg.MODEL.IMG_SIZE = 384
    cfg.MODEL.IS_EXTRACT = True
    cfg.DATASET.TEST_SET = 'test'

    model = build_network(cfg.MODEL)
    model.load_state_dict(torch.load(load_checkpoint))

    model = model.to(device)
    model.eval()

    return model